/***********************************************************
* DMX-512 Reception                                        *
* Developed by Max Pierson                                 *
* Version Rev09 20 March 2009                              *
* Released under the WTFPL license, although I would       *
* appreciate Attribution and Share-Alike                   *
* See blog.wingedvictorydesign.com for the latest version. *
************************************************************/


/******************************* MAX485 variable declarations ***********************************/

int receiveroutputenable = 3;   
/* receiver output enable (pin2) on the max485.  
*  will be left low to set the max485 to receive data. */

int driveroutputenable = 4;     
/* driver output enable (pin3) on the max485.  
*  will left low to disable driver output. */

int rxpin = 0;   // serial receive pin, which takes the incoming data from the MAX485.
int txpin = 1;   // serial transmission pin

/******************************* Serial variable declarations ***********************************/


unsigned long baud = 250000;    // The baud rate for DMX is 250000bps, since each bit = 4 microseconds.
unsigned long clockspeed = F_CPU; 
/* F_CPU is a predefined constant equal to the clock speed of the microcontroller.
*  for the Arduino duomilanove it is 16000000.
*  I'm going to dump it into clockspeed because I'm worried about it being a signed long,
*  which could screw up the bitwise math I'm going to do on baudprescale, maybe. */

unsigned long baudprescale;     
/* The baud prescale that will be loaded into the 
*  UBRR0H and UBRR0L registers to set the USART baud rate */

/******************************* DMX variable declarations ***********************************/

volatile byte i = 0;               //dummy variable for dmxvalue[]

volatile byte dmxreceived = 0;     //the latest received value
volatile byte dmxcurrent = 0;      //counter variable that is incremented every time we receive a value.

volatile byte dmxvalue[8] = {0, 0, 0, 0, 0, 0, 0, 0};     //stores the DMX values we're interested in using.
byte dmxaddress = 37;              //the dmx address we want to listen to.

volatile boolean dmxnewvalue = 0;     //set to 1 when new dmx values are received.

/******************************* Timer2 variable declarations ***********************************/

volatile byte zerocounter = 0;          
/* a counter to hold the number of zeros received in sequence on the serial receive pin.  
*  When we've received a minimum of 11 zeros in a row, we must be in a break.  As written,
*  the timer2 ISR actually checks for 22 zeros in a row, for the full 88uS break.       */

void setup() {
  
  /******************************* Max485 configuration ***********************************/
  
  pinMode(receiveroutputenable, OUTPUT);
  pinMode(driveroutputenable, OUTPUT);
  digitalWrite(receiveroutputenable, LOW);
  digitalWrite(driveroutputenable, LOW);    //sets pins 3 and 4 to low to enable reciever mode on the MAX485.

  pinMode(rxpin, INPUT);  //sets serial pin to receive data

  /******************************* USART configuration ***********************************/

  baudprescale = (((clockspeed / (baud * 16UL))) - 1);
  /* calculates the baud rate prescale that will be loaded into the UBRR0L and UBRR0H registers.
  *  See page 173 of the datasheet for this calculation. */

  cli();  //disable interrupts while initializing the USART and Timer2

  bitSet(UCSR0A, TXC0);
  bitClear(UCSR0A, FE0);
  bitClear(UCSR0A, DOR0);
  bitClear(UCSR0A, UPE0);
  bitClear(UCSR0A, U2X0); 
  bitClear(UCSR0A, MPCM0);
  /* set transfer complete flag (TXC0 = 1).
  *  clear Frame Error flag (FE0 = 0).   
  *  clear Data overrun flag (DOR0 = 0).
  *  clear Parity mismatch flag (UPE0 = 0).
  *  disable doubling of USART reception/transmission speed (U2X0 = 0).  
  *  Disable Multi-Processor Communication Mode-- whatever that is. (MCPM0 = 0)  */
  
  bitClear(UCSR0B, RXCIE0);
  bitClear(UCSR0B, TXCIE0);
  bitClear(UCSR0B, UDRIE0);
  bitSet(UCSR0B, RXEN0);
  bitSet(UCSR0B, TXEN0);
  bitClear(UCSR0B, UCSZ02);
  /* Disable Receive Interrupt (RXCIE0 = 1).
  *  Disable Tranmission Interrupt (TXCIE = 0).
  *  Disable Data Register Empty interrupt (UDRIE0 = 0). 
  *  Enable reception (RXEN0 = 1).
  *  Enable transmission (TXEN0 = 1). 
  *  Set 8-bit character mode (UCSZ00, UCSZ01, and UCSZ02 together control this, 
  *  But UCSZ00, UCSZ01 are in Register UCSR0C). */
  
  bitClear(UCSR0C, UMSEL00);
  bitClear(UCSR0C, UMSEL01);
  bitClear(UCSR0C, UPM00);
  bitClear(UCSR0C, UPM01);
  bitSet(UCSR0C, USBS0); 
  bitSet(UCSR0C, UCSZ00);
  bitSet(UCSR0C, UCSZ01);
  bitClear(UCSR0C, UCPOL0);
  /* USART Mode select -- UMSEL00 = 0 and UMSEL01 = 0 for asynchronous mode.
  *  disable parity mode -- UPM00 = 0 and UPM01 = 0. 
  *  Set USBS = 1 to configure to 2 stop bits per DMX standard.  The USART receiver ignores this 
  *  setting anyway, and will only set a frame error flag if the first stop bit is 0.  
  *  But, we have to set it to something.
  *  Finish configuring for 8 data bits by setting UCSZ00 and UCSZ01 to 1.  
  *  Set clock parity to 0 for asynchronous mode (UCPOL0 = 0). */
  
  UBRR0L = baudprescale; // Load lower 8-bits of the baud rate value into the low byte of the UBRR register 
  UBRR0H = (baudprescale >> 8); // Load upper 8-bits of the baud rate value into the high byte of the UBRR register 
  // ***note to self: at some point add a command to write UBRR0H high bits to 0 per datasheet for "future compatibility"
  
  dmxaddress = dmxaddress + 3;
  /*  this will allow the USART receive interrupt to fire an additional 3 times.  Here's why:
  *   Once for the last value received by the USART before the break, which will still be in the USART buffer, 
  *         causing the receive ISR to execute immediately upon enabling it,
  *   Once for the Mark After Break (MAB), which will be detected by the USART as a valid character 
  *        (eight zeros followed by a one)
  *   Once for the START code that precedes the 512 DMX values (used for RDM).  */
  
  
  /******************************* Timer2 configuration ***********************************/
  
  //NOTE:  this will disable PWM on pins 3 and 11.
  bitClear(TCCR2A, COM2A1);
  bitClear(TCCR2A, COM2A0); //disable compare match output A mode
  bitClear(TCCR2A, COM2B1);
  bitClear(TCCR2A, COM2B0); //disable compare match output B mode
  bitSet(TCCR2A, WGM21);
  bitClear(TCCR2A, WGM20);  //set mode 2, CTC.  TOP will be set by OCRA.
  
  bitClear(TCCR2B, FOC2A);
  bitClear(TCCR2B, FOC2B);  //disable Force Output Compare A and B.
  bitClear(TCCR2B, WGM22);  //set mode 2, CTC.  TOP will be set by OCRA.
  bitClear(TCCR2B, CS22);
  bitClear(TCCR2B, CS21);
  bitSet(TCCR2B, CS20);   // no prescaler means the clock will increment every 62.5ns (assuming 16Mhz clock speed).
  
  OCR2A = 64;                
  /* Set output compare register to 64, so that the Output Compare Interrupt will fire
  *  every 4us.  */
  
  bitClear(TIMSK2, OCIE2B);  //Disable Timer/Counter2 Output Compare Match B Interrupt
  bitSet(TIMSK2, OCIE2A);    //Enable Timer/Counter2 Output Compare Match A Interrupt
  bitClear(TIMSK2, TOIE2);   //Disable Timer/Counter2 Overflow Interrupt Enable          
  
  sei();                     //reenable interrupts now that timer2 has been configured. 
  
}  //end setup()



void loop()  {
  // the processor gets parked here while the ISRs and action() loops are doing their thing. 
  
  if (dmxnewvalue == 1) {    //when a new value is received, jump to action loop...
    action();
    dmxnewvalue = 0;
    dmxcurrent = 0;
    zerocounter = 0;      //and then when finished reset variables and enable timer2 interrupt
    i = 0;
    bitSet(TIMSK2, OCIE2A);    //Enable Timer/Counter2 Output Compare Match A Interrupt
  }
} //end loop()



/********************** Put what you want the code to do with the values (dmxvalue) here ******************/
void action() {
byte j;
for(j = 0; j < 8; j++) {
  Serial.print(dmxvalue[j]);
}
  
  analogWrite(6, dmxvalue[0]);
  analogWrite(5, dmxvalue[1]);
 
  return;                          //go back to loop()
} //end action() loop



//Timer2 compare match interrupt vector handler
ISR(TIMER2_COMPA_vect) {
  if (bitRead(PIND, PIND0)) {  // if a one is detected, we're not in a break, reset zerocounter.
    zerocounter = 0;
    }
  else {
    zerocounter++;             // increment zerocounter if a zero is received.
    if (zerocounter == 22) {
      bitClear(TIMSK2, OCIE2A);    //disable this interrupt and enable reception interrupt now that we're in a break.
      bitSet(UCSR0B, RXCIE0);
      }
  }
}    //end Timer2 ISR



ISR(USART_RX_vect){
  dmxreceived = UDR0;
  /* The receive buffer (UDR0) must be read during the reception ISR, or the ISR will just 
  *  execute again immediately upon exiting. */
 
  dmxcurrent++;                        //increment address counter
 
  if(dmxcurrent > dmxaddress) {         //check if the current address is the one we want.
    dmxvalue[i] = dmxreceived;
    i++;
    if(i == 8) {
      bitClear(UCSR0B, RXCIE0); 
      dmxnewvalue = 1;                        //set newvalue, so that the main code can be executed.
    } 
  }
} // end ISR
